this.name        = "reversecontrol";
this.author      = "Norby";
this.copyright   = "2014 Norbert Nagy";
this.licence     = "CC BY-NC-SA 3.0";
this.description = "Reverse up-down control in the aft view.";

(function(){
	"use strict";

//customizable properties
this.$ReverseControlOn = true; //can turn off without uninstall

//internal properties, should not touch
this.$ReverseControlFCB = null; //FrameCallBack

//world script events
this.step1Up = function() {
}

this.shipWillDockWithStation = function(/*station*/) {
	this.$Stop();
}

this.viewDirectionChanged = function( view ) {
	switch ( view ) {
		case "VIEW_AFT":
		case "VIEW_PORT":
		case "VIEW_STARBOARD":
			if( this.$ReverseControlOn && !isValidFrameCallback( this.$ReverseControlFCB ) )
				this.$ReverseControlFCB = addFrameCallback( this.$ReverseControl_FCB.bind(this) );
			break;
		default:
			this.$Stop();
	}
}


// moved code in from core so this oxp no longer generates any garbage to be collected (speed is comparable)
this._rotate_about_axis = function _rotate_about_axis( quat, vector, angle, result ) {
	var that = _rotate_about_axis;
	var cos = (that.cos = that.cos || Math.cos);
	var sin = (that.sin = that.sin || Math.sin);
	var rotn = (that.rotn = that.rotn || [0, 0, 0, 0]);

	var a = angle / 2;
	var c = cos(a);
	var s = sin(a);
	// rotation quaternion
	rotn[0] = c;
	rotn[1] = vector[0] * s;
	rotn[2] = vector[1] * s;
	rotn[3] = vector[2] * s;
	// multiply quaternions
	result[0] = quat[0] * rotn[0] - quat[1] * rotn[1] - quat[2] * rotn[2] - quat[3] * rotn[3];
	result[1] = quat[0] * rotn[1] + quat[1] * rotn[0] + quat[2] * rotn[3] - quat[3] * rotn[2];
	result[2] = quat[0] * rotn[2] + quat[2] * rotn[0] + quat[3] * rotn[1] - quat[1] * rotn[3];
	result[3] = quat[0] * rotn[3] + quat[3] * rotn[0] + quat[1] * rotn[2] - quat[2] * rotn[1];
}

// 		console.script.wr = worldScripts.reversecontrol;
// 		console.script.ps = player.ship;

/*		(function() {
var wr = worldScripts.reversecontrol;
if( isValidFrameCallback( wr.$ReverseControlFCB ) )
	removeFrameCallback( wr.$ReverseControlFCB );
wr.$ReverseControlFCB = addFrameCallback( wr.$ReverseControl_FCB.bind(wr) );

})()
//*/

//ReverseControl methods
this.$ReverseControl_FCB = function ReverseControl_FCB( delta ) { //FrameCallBack
	var that = ReverseControl_FCB;
	var wr = (that.wr = that.wr || worldScripts.reversecontrol);
	var _rotate_about_axis = (that._rotate_about_axis = that._rotate_about_axis || wr._rotate_about_axis);
	var step1 = (that.step1 = that.step1 || []);
	var step2 = (that.step2 = that.step2 || []);
	var prevVR = (that.prevVR = that.prevVR || null);
	var prevVF = (that.prevVF = that.prevVF || null);

    var a, ps = player.ship;
    if( ps && ps.isValid && !ps.docked && this.$ReverseControlOn && ps.AI != "dockingAI.plist" ) {
		if( prevVF === null ) { // 1st time thru
			prevVF = ps.vectorForward;
			prevVR = ps.vectorRight;
		}
		switch ( ps.viewDirection ) {
			case "VIEW_AFT":
				a = ps.pitch * delta;
				_rotate_about_axis(ps.orientation, prevVR, 2*a, step1);	// 2 * incl's correcting for cmd that got us here
				a = ps.roll * delta;
				_rotate_about_axis(step1, prevVF, 2*a, step2);			
				ps.orientation = step2;
				break;
			case "VIEW_PORT": 	
				// important to alternate prevVR, prevVF,...; doing prevVR, prevVF, prevVF, prevVR as before
				// will cause center to be off vertically; but this way, only set ps.orientation once, not 4 times
				//pitch -> roll
				a = ps.pitch * delta;
				_rotate_about_axis(ps.orientation, prevVR,  a, step1);	// - undo this frame's pitch
				_rotate_about_axis(step1, prevVF, -a, step2);			// - apply pitch cmd as roll		
				//roll -> -pitch
				a = ps.roll * delta;
				_rotate_about_axis(step2, prevVR, a, step1);			// - apply roll cmd as pitch
				_rotate_about_axis(step1, prevVF, a, step2);			// - undo this frame's roll
				ps.orientation = step2;
				break;
			case "VIEW_STARBOARD":
				// important to alternate prevVR, prevVF,...; doing prevVR, prevVF, prevVF, prevVR as before
				// will cause center to be off vertically; but this way, only set ps.orientation once, not 4 times
				//pitch -> roll
				a = ps.pitch * delta;
				_rotate_about_axis(ps.orientation, prevVR,  a, step1); 	// - undo this frame's pitch
				_rotate_about_axis(step1, prevVF,  a, step2);			// - apply pitch cmd as roll			
				//roll -> -pitch
				a = ps.roll * delta;
				_rotate_about_axis(step2, prevVR, -a, step1);			// - apply roll cmd as pitch
				_rotate_about_axis(step1, prevVF,  a, step2);			// - undo this frame's roll			
				ps.orientation = step2;
				break;
			default:
				prevVF = prevVR = null;
				this.$Stop();
		}
		prevVF = ps.vectorForward;
		prevVR = ps.vectorRight;
    } else {
		prevVF = prevVR = null;
		this.$Stop();
    }
}

this.$Stop = function() {
	var rcFCB = this.$ReverseControlFCB;
	if( rcFCB && isValidFrameCallback( rcFCB ) ) {
		removeFrameCallback( rcFCB );
		this.$ReverseControlFCB = null;
	}
}

/*
this.$ReverseControl_FCB = function( delta ) { //FrameCallBack, improved by cag
    var ps = player.ship;
    var a, c = null;
    if( ps && ps.isValid && !ps.docked && this.$ReverseControlOn && ps.AI != "dockingAI.plist" ) {
	var prevori = this.$ReverseControlPrevOri;
	if( prevori ) switch ( ps.viewDirection ) {
	    case "VIEW_AFT":
		//reverse up-down
		a = ps.pitch * delta;
		c = prevori.vectorRight();
		ps.orientation = ps.orientation.rotate( c, 2 * a ); // 2 * incl's correcting for pitch cmd that got us here
		//reverse roll
		a = ps.roll * delta;
		c = prevori.vectorForward();
		ps.orientation = ps.orientation.rotate( c, 2 * a ); // 2 * incl's correcting for roll cmd that got us here
		break;
	    case "VIEW_PORT":
		//pitch -> roll
		a = ps.pitch * delta;
		c = prevori.vectorRight();
		ps.orientation = ps.orientation.rotate( c, 1 * a ); // - 1st undo this frame's pitch
		c = prevori.vectorForward();
		ps.orientation = ps.orientation.rotate( c, -1 * a ); // - now apply pitch cmd as roll
		//roll -> -pitch
		a = ps.roll * delta;
		c = prevori.vectorForward();
		ps.orientation = ps.orientation.rotate( c, 1 * a ); // - 1st undo this frame's roll
		c = prevori.vectorRight();
		ps.orientation = ps.orientation.rotate( c, 1 * a ); // - now apply roll cmd as pitch
		break;
	    case "VIEW_STARBOARD":
		//pitch -> roll
		a = ps.pitch * delta;
		c = prevori.vectorRight();
		ps.orientation = ps.orientation.rotate( c, 1 * a ); // - 1st undo this frame's pitch
		c = prevori.vectorForward();
		ps.orientation = ps.orientation.rotate( c, 1 * a ); // - now apply pitch cmd as roll
		//roll -> -pitch
		a = ps.roll * delta;
		c = prevori.vectorForward();
		ps.orientation = ps.orientation.rotate( c, 1 * a ); // - 1st undo this frame's roll
		c = prevori.vectorRight();
		ps.orientation = ps.orientation.rotate( c, -1 * a ); // - now apply roll cmd as pitch
		break;
	    default:
		this.$Stop();
	}
	this.$ReverseControlPrevOri = ps.orientation;
    } else {
	this.$ReverseControlPrevOri = null;
    }
}

this.$ReverseControl_FCB_old = function( delta ) { //FrameCallBack, old version
	var ps = player.ship;
	if( ps && ps.isValid && !ps.docked && this.$ReverseControlOn && ps.AI != "dockingAI.plist" ) {
		var prevori = this.$ReverseControlPrevOri;
		if( prevori ) switch ( ps.viewDirection ) {
			case "VIEW_AFT":
				//reverse up-down
				//var a = ps.heading.angleTo( prevori.vectorUp() ) - Math.PI / 2; //buggy with sniperlock
				var a = ps.pitch * delta;
				var c = prevori.vectorRight();
				ps.orientation = ps.orientation.rotate( c, 2 * a );
				//reverse roll
				//var a = ps.orientation.vectorRight().angleTo( prevori.vectorUp() ) - Math.PI / 2; //buggy with sniperlock
				var a = ps.roll * delta;
				var c = prevori.vectorForward();
				ps.orientation = ps.orientation.rotate( c, 2 * a );
				break;
			case "VIEW_PORT":
				//pitch -> roll
				var a = ps.pitch * delta;
				var c = prevori.vectorForward();
				ps.orientation = ps.orientation.rotate( c, -2 * a );
				//roll -> -pitch
				var a = ps.roll * delta;
				var c = prevori.vectorRight();
				ps.orientation = ps.orientation.rotate( c, 2 * a );
				break;
			case "VIEW_STARBOARD":
				//pitch -> -roll
				var a = ps.pitch * delta;
				var c = prevori.vectorForward();
				ps.orientation = ps.orientation.rotate( c, 2 * a );
				//roll -> pitch
				var a = ps.roll * delta;
				var c = prevori.vectorRight();
				ps.orientation = ps.orientation.rotate( c, -2 * a );
				break;
			default:
				this.$Stop();
		}
		this.$ReverseControlPrevOri = ps.orientation;
	} else {
		this.$ReverseControlPrevOri = null;
	}
}
*/

}).call(this);
